"""
    QuaO Project http_header.py Copyright © CITYNOW Co. Ltd. All rights reserved.
"""
from ..enum.base_enum import BaseEnum


class HttpHeader(BaseEnum):
    AUTHORIZATION = 'Authorization'
    CONTENT_TYPE = 'Content-type'
    ACCEPT = 'Accept'

